/*


	This program solves pump test data using the slope matching methods
	It is compiled using Borland C++ 5.0, however the code should compile
  on almost any C/C++ compiler

	Data files created with this program are readable by Starpoint
	Software's Infinite Extent Pump Test Program for Windows

	written by Robert Pittenger


*/

/////////////////////////////////////////////////////////////////////
/*
	Notes for FORTRAN Programmers

	This program is written in C using input and output features of
	C++.  Although the basic logic of the program can be followed
	without knowing much about C, several differences are important.
	Some comparisons to FORTRAN are commented throughout the program

	The most important difference is in the subscripting of arrays
	In C an array called x with four elements would be declared:

	int x[4];

	The elements would be number 0 through 3.  The value x[4] is not valid.
	Array subscripts are also enclosed in brackets [] rather than parentheses

	Also in C, every statement ends with a semi-colon.

	Variable names in C are case sensitive

	Functions declared as type 'void' do not return a value
	Functions with 'void' in the parameter list do not accept parameters

  C comments like this one are enclosed in slash-asterisk pairs
  C++ comments are preceeded by // and only comment out the line
  containing the //

*/


// include head files for standard C++ libraries
#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <iomanip.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>

// define constants for the program
#define MAX 200 // the maximum number of trials allowed
#define MAXFILELENGTH 80 // max length of a file name


// this is a structure whichs contains the pump test data
typedef struct
{
	char Title[80]; // a description of the test
	float PumpingRate; // the well discharge rate
	float Thickness; // the aquifer thickness
	float Radius;  // the distance to the piezometer in which drawdown is
								 // measured
	float Drawdown[MAX]; // the drawdown in the piezometer
	float Time[MAX]; // the time at which drawdown is measured
	int MaxTrials; // the total number of drawdown measurements
} WELLDATA;


// declare a global variable wd of structure WELLDATA
WELLDATA wd;

// these global variables are used for the calculation results
// the names are similar to those in the Table by Sen (1986)
double Delta[MAX],	// change from steady-state drawdown for the Hantush method
							 // this is called 'Drawdown Difference' in Sen Table 5
			Slope[MAX], // the slope of the Theis curve
			q[MAX], // r*r*S/4Tt, sometime called 'u'
			Wq[MAX], // Theis well function
			KoRL[MAX], // the value of Ko(r/L) for the Hantush method
			RL[MAX], // the value of r/L for the Hantush method
			T[MAX], // Transmissivity calculated by slope matching
			S[MAX], // Storativity calculated by slope matching
			L[MAX]; // Leakage factor calculated with Hantush method



// these are the basic math functions for use in the program


//  calculates factorial of arg by Brute Force
double fac(int n)
{
	switch (n)
	{
		case 1:
			return(1.0);
		case 2:
			return(2.0);
		case 3:
			return(6.0);
		case 4:
			return(24.0);
		case 5:
			return(120.0);
		case 6:
			return(720.0);
		case 7:
			return(5040.0);
		case 8:
			return(40320.0);
		case 9:
			return(362880.0);
		case 10:
			return(3628800.0);
		case 11:
			return(39916800.0);
		case 12:
			return(479001600.0);
		case 13:
			return(6227020800.0);
		default:
			return(1);
  }
}// end of factorial


// function to calculate factorial up to 15!
long double facl(int n)
{
	switch (n)
	{
		case 1:
			return(1.0);
		case 2:
			return(2.0);
		case 3:
			return(6.0);
		case 4:
			return(24.0);
		case 5:
			return(120.0);
		case 6:
			return(720.0);
		case 7:
			return(5040.0);
		case 8:
			return(40320.0);
		case 9:
			return(362880.0);
		case 10:
			return(3628800.0);
		case 11:
			return(39916800.0);
		case 12:
			return(479001600.0);
		case 13:
			return(6227020800.0);
		case 14:
			return(87178291200.0);
    case 15:
			return(1307674368000.0);
		default:
			return(1);
  }
}// end of factorial




double ln(double arg)
{
	return(log(arg));
}




// calcualtes Theis well function for hydrogeology
double WellFunction(double u)
{
	if (u <= 0.0) return(0.0); // check for errors
	double val;
	val = -0.5772 - log(u) + u - pow(u, 2.0)/(2.0*fac(2)) + pow(u,3.0)/(3.0*fac(3))
		- pow(u,4.0)/(4.0*fac(4)) + pow(u,5.0)/(5.0*fac(5)) - pow(u,6.0)/(6.0*fac(6))
		+ pow(u,7.0)/(7.0*fac(7)) - pow(u,8.0)/(8.0*fac(8))
		+ pow(u,9.0)/(9.0*fac(9)) -	pow(u,10.0)/(10.0*fac(10))
		+ pow(u,11.0)/(11.0*fac(11)) - pow(u,12.0)/(12.0*fac(12));
	return (val);
}

// calcualtes Theis well function for hydrogeology using long doubles
long double WellFunctionl(long double u)
{
	if (u <= 0.0) return(0.0); // check for errors
	long double val;
	val = -0.5772 - logl(u) + u - powl(u, 2.0)/(2.0*fac(2)) + powl(u,3.0)/(3.0*fac(3))
		- powl(u,4.0)/(4.0*fac(4)) + powl(u,5.0)/(5.0*fac(5)) - powl(u,6.0)/(6.0*fac(6))
		+ powl(u,7.0)/(7.0*fac(7)) - powl(u,8.0)/(8.0*fac(8))
		+ powl(u,9.0)/(9.0*fac(9)) -	powl(u,10.0)/(10.0*fac(10))
		+ powl(u,11.0)/(11.0*fac(11)) - powl(u,12.0)/(12.0*fac(12))
		+ powl(u,13.0)/(13.0*fac(13)) - powl(u,14.0)/(14.0*facl(14))
    + powl(u,15.0)/(15.0*facl(15));
	return (val);
}



double GetU(double slope)
//  This function calculates u from slope matching
// it returns a zero if it is out of calcuable range
// slope MUST be positive coming into the function
// therefore,the Theis method should reverse the sign of the slope
// before calling the function, and the Hantush method should send
//slope at it's original sign
// u is r*r*S/(4*T*t)
{

	double guess, u, logu;
	int counter = 0;
	double err = .1;
	double lowerbound = -10.0;
	double upperbound = log10(3.8);

	if (slope < 0.0) return(0.0);
	if (slope > pow(10.0, lowerbound) && slope < pow(10.0, upperbound))
	{
		logu = -5.0;
		u = pow(10.0, logu);
		guess = -exp(-u)/WellFunction(u);
		if (guess < 0.0) guess = -guess;
		do
		{
			if (guess < slope) // move right
			{
				lowerbound = logu;
				logu = (upperbound+lowerbound)/2.0;
			}
			if (guess > slope) // move left
			{
				upperbound = logu;
				logu = (upperbound+lowerbound)/2.0;
			}
			u = pow(10.0, logu);
			err = fabs(slope - guess);
			guess = -exp(-u)/WellFunction(u);
			if (guess < 0.0) guess = -guess;
			++counter;
		}
		while (err > .00001 && counter < 500);
	}  // end if
	else u = 0.0;
	return(u);
} // end of function





double bessio(double x)
// returns the modified bessel function Io(x) for any real x
/*
	This function is copied from:
	Press, William, H., Saul A. Teukolsky, William T. Vetterling,
	and Brian P. Flannery. 1992. 'Numerical recipes in C:
	The art of scientific computing,' Cambridge University Press.
	and is subject to their copyright restriction
*/

{
	double ax, ans, y;

	ax = fabs(x);
	if (ax < 3.75)
	{
		y = x / 3.75;
		y*=y;
		ans = 1.0+y*(3.5156229+y*(3.0899424+y*(1.2067492
			+y*(0.2659732+y*(0.360768e-1+y*0.45813e-2)))));
	}
	else
	{
		y = 3.75/ax;
		ans = (exp(ax)/sqrt(ax))*(0.39894228+y*(0.1328592e-1
			+y*(0.225319e-2+y*(-0.157565e-2+y*(0.916281e-2
			+y*(-0.2057706e-1+y*(0.2635537e-1+y*(-0.1647633e-1
			+y*0.392377e-2))))))));
	}
	return(ans);
}




double bessko(double x)
// returns the modified Bessel function Ko for positive real x
/*
	This function is copied from:
	Press, William, H., Saul A. Teukolsky, William T. Vetterling,
	and Brian P. Flannery. 1992. 'Numerical recipes in C:
	The art of scientific computing,' Cambridge University Press.
	and is subject to their copyright restriction
*/
{
	double bessio(double x);
	double y, ans;
	if (x <= 2.0)
	{
		y = x*x/4.0;
		ans = (-log(x/2.0)*bessio(x))+(-0.57721566+y*(0.42278420
			+y*(0.23069756+y*(0.3488590e-1+y*(0.262698e-2
			+y*(0.10750e-3+y*0.74e-5))))));
	}
	else
	{
	y = 2.0/x;
	ans = (exp(-x)/sqrt(x))*(1.25331414+y*(-0.7832358e-1
		+y*(0.2189568e-1+y*(-0.1062446e-1+y*(0.587872e-2
		+y*(-0.251540e-2+y*0.53208e-3))))));
	}
	return(ans);
}  // end of bessko





// this functions returns the value x for a specified y for
// the equation y = Ko(x)  where Ko is a bessel function
float GetKoArg(float check)
{
	float x = 1, max = 1, min = 0, error = 1, result;
	do
	{
		result = bessko(max);
		if (result > check)
			max = max + 1;
	}
	while (result > check);
	do
	{
		result = bessko(x);

		if (result < check) // decrease x
		{
			max = x;
			x = (x + min)/2.0;
		}
		else // increase x
		{
			min = x;
			x = (x + max)/ 2.0;
		}
		error = fabs(result-check);
	}
	while (error > 0.00001);
	return(x);
}




void InitializeValues(void)
{ // this function sets all the values to 0
	int counter;
	for (counter = 0; counter < MAX; ++counter)
	{
		S[counter] = 0.0;
		T[counter] = 0.0;
		Delta[counter] = 0.0;
		Slope[counter] = 0.0;
		q[counter] = 0.0;
		Wq[counter] = 0.0;
		KoRL[counter] = 0.0;
		RL[counter] = 0.0;
	}
} // end of initialize






void TheisCalculator(void)
{ // this function does the slope matching calculations
	// using the Theis method for confined aquifers
	// results are assigned to global variables
	int counter;
	double Alpha, u, Wu;

	cout<<"\nCalculating T and S with Theis slope matching\n";
	cout<<"Trial\tTime\tDrawdown\tT\tS"<<endl;
	// start with drawdown 1, for n measurements, there will be
	// n-1 transmissivity and storativity values
	// calculate using forward difference method
	for (counter = 0; counter < wd.MaxTrials; ++counter)
	{
		if (wd.Drawdown[counter] != 0.0	&& wd.Time[counter+1] != 0.0)
		{  // this condition prevents divide by zero in the Alpha calculation
			 // this is Equation 5  which calculates the slope of the data

			// calculate the slope of the data points using forward
      // difference method
			Alpha = log(wd.Drawdown[counter+1]/wd.Drawdown[counter])/
					log(wd.Time[counter+1]/wd.Time[counter]);

			Slope[counter] = Alpha; // assign to the global array

			// use the get_u function to find the value of u
			// Alpha should be positive for positive drawdown
      // although the Sen paper used negative slope values
			// GetU must accept a positive number
			// Now if drawdown decreased as time increased, GetU will
			// recieve a negative number and return a 0.  This is what we want.
			// but don't call the well function with a 0!
			u = GetU(Alpha);
			q[counter] = u; 			// assign to the global array
			if (u != 0.0) Wu = WellFunction(u); // don't call Well Function with 0 for u
			Wq[counter] = Wu; 			// assign to global variable array
			if(u > 0.0 && Wu > 0.0)   // prevent divide by 0
			{
				// calculate transmissivity using Equation 1
				T[counter] = (Wq[counter]*wd.PumpingRate)/
					(12.56637061*wd.Drawdown[counter]);
				// Calculate storativity from Equation 2
				S[counter] = (4.0 * wd.Time[counter] * T[counter] * q[counter])/
					 (wd.Radius*wd.Radius);
			}  // end of if
			else // for bad data, assign a negative number corresponding to
			{   // where the data went bad
				T[counter] = -1;
				S[counter] = -1; // u or Wu was zero
			} // end of else
		}  //end if
		else
		{
			T[counter] = -2; // ime[counter] == 0 or
			S[counter] = -2; // drawdown[counter - 1] == 0
		}  // end of else


		// write results to the screen
		cout<<(counter+1)<<"\t"<<wd.Time[counter]<<"\t"<<wd.Drawdown[counter]
			<<"\t"<<T[counter]<<"\t"<<S[counter]<<endl;
	} // end of loop
	cout<<endl;
}  // end of TheisCalculator





void HantushCalculator(void)
{ // This functions performs the slope matching calculations
	// for leaky aquifers using the Hantush method
	// results are assigned to global variables

	int counter, BadData; // BadData is a flag used to prevent divide by zero
	double Alpha, // temporary storage of data slopes
				 u, // temporary storage of u
				 Wu, // temporary storage of W(u)
				 temp, // temporary storage variable
				 SSDraw, // steady state drawdown
				 arg; // temporary variable for argument to log funtion

	// make the last drawdown value the steady-state drawdown
	SSDraw = wd.Drawdown[wd.MaxTrials-1];
	cout<<"\nCalculating T, S, and L with Hantush slope matching\n";
	cout<<"Trial\tTime\tDrawdown\tT\tS\tL"<<endl;

	for (counter = 0; counter < wd.MaxTrials-1; ++counter)
	{
		BadData = 0;
		// calculate difference from steady state drawdown
		Delta[counter] = SSDraw - wd.Drawdown[counter];
		Delta[counter+1] = SSDraw - wd.Drawdown[counter+1];

		// check for condtions that will cause math errors and
		// assign a value to BadData indicate the type of error
		if (wd.Time[counter+1] == 0.0) BadData = -1;
		if (wd.Time[counter] == 0.0) BadData = -2;
		if (wd.Drawdown[counter+1] >= SSDraw) BadData = -3;
		if (wd.Drawdown[counter] >= SSDraw) BadData = -4;

		if (BadData == 0)
		{
			// Calculate the slope of the field data (Equation 10)
			arg = (SSDraw - wd.Drawdown[counter])/(SSDraw - wd.Drawdown[counter+1]);
      if (arg > 1e-15)
				Alpha = log(arg)/log(wd.Time[counter+1]/wd.Time[counter]);
			else BadData = -5;
					Slope[counter] = Alpha;

			u = GetU(Alpha);
			q[counter] = u; // assign u to glabal array
			if (u != 0.0) Wu = WellFunction(u);
			Wq[counter] = Wu; // assign W(u) to global array
			if(u > 0.0 && Wu > 0.0)   // prevent divide by 0
			{
				// calculate T with equation 1
				T[counter] = (Wu*wd.PumpingRate)/
					(12.56637061*(SSDraw - wd.Drawdown[counter]));
				// calculate S with equation 2
        // this is wrong
//				S[counter] = (4.0 * wd.Time[counter]*T[counter]*u)/
//					(wd.Radius * wd.Radius);

				// calculate Leakage with equation 8
				temp = 6.2131853*T[counter]*SSDraw/wd.PumpingRate;
				KoRL[counter] = temp;
				RL[counter] = GetKoArg(temp);
				L[counter] = wd.Radius/RL[counter];

				// calculate S with equation 7
        S[counter] = T[counter]*wd.Time[counter]/
        	(L[counter]*L[counter]*u);
			}
			else
			{
				T[counter] = -6; // assign an error flag to the values
				S[counter] = -6; // here u or W(u) was equal to zero
				L[counter] = -6;
			} // end of else
		}  //end if
		else
		{
			T[counter] = BadData; // assign error flag to values
			S[counter] = BadData; // to diagnose why method failed
			L[counter] = BadData;
		}
		cout<<(counter)<<"\t"<<wd.Time[counter]<<"\t"<<wd.Drawdown[counter]
			<<"\t"<<T[counter]<<"\t"<<S[counter]<<"\t"<<L[counter]<<endl;
	}  // end of loop for counter
	cout<<endl;
}  // end of HantushCalculator






void WriteTheisFile(void)
{  // This functions writes a file of the calculation results
	// from the Theis slope matching method.  The table is formatted
	// like Table 1 in the paper.  It is tab delimited for easy
	// importing to a spreadsheet
	ofstream fcout;
	int counter;
	char FileName[MAXFILELENGTH];

	cout<<"\nEnter the Output Filename ";
	cin>>FileName;
	fcout.open(FileName);
	fcout<<"Time\tDrawdown\tSlope\tu\tW(u)\tTransmissivity\tStor"<<endl;

	for (counter = 0; counter < wd.MaxTrials; ++counter)
	{
		fcout<<setiosflags(ios::scientific)<<wd.Time[counter]<<"\t";
		fcout<<setiosflags(ios::fixed)<<wd.Drawdown[counter]<<"\t";
		fcout<<Slope[counter]<<"\t";
		fcout<<setiosflags(ios::scientific)<<q[counter]<<"\t";
		fcout<<setiosflags(ios::fixed)<<Wq[counter]<<"\t";
		fcout<<T[counter]<<"\t";
		fcout<<setiosflags(ios::scientific)<<S[counter]<<"\t";
		fcout<<setiosflags(ios::fixed);
		fcout<<endl;
	}
	cout<<endl;
} // end of function WriteTheisFile




void WriteHantushFile(void)
{ // This function creates a table of calculation results for
	// the Hantush slope-matching method.  Results are formatted
	// as they appear in Table 2 in the paper.  Values are
	// tab delimited for importing into a spreadsheet
	ofstream fcout;
	int counter;
	char FileName[MAXFILELENGTH];

	cout<<"\nEnter the output filename ";
	cin>>FileName;
	fcout.open(FileName);
	fcout<<"Time\tDrawdown\tDelta\tSlope\tq\tW(q)\tTrans\tKo(r/L)\tr/L\tLeakage\tStor"<<endl;
	for (counter = 0; counter < wd.MaxTrials; ++counter)
	{
		fcout<<setiosflags(ios::scientific)<<wd.Time[counter]<<"\t";
		fcout<<setiosflags(ios::fixed)<<wd.Drawdown[counter]<<"\t";
		fcout<<Delta[counter]<<"\t";
		fcout<<Slope[counter]<<"\t";
		fcout<<setiosflags(ios::scientific)<<q[counter]<<"\t";
		fcout<<setiosflags(ios::fixed)<<Wq[counter]<<"\t";
		fcout<<T[counter]<<"\t";
		fcout<<KoRL[counter]<<"\t";
		fcout<<RL[counter]<<"\t";
		fcout<<L[counter]<<"\t";
		fcout<<setiosflags(ios::scientific)<<S[counter]<<"\t";
		fcout<<setiosflags(ios::fixed);
		fcout<<endl;
	}
	cout<<endl;
} // end of function WriteHantushFile





void ViewData(void)
{
	// this function displays all of the test data on the screen
	// it will not display more that 10 lines without a user prompt
	// this prevents scrolling all of the data

	div_t val; // this type requires the header file stdlib.h
	int LineCount = 6, counter;
	char TempChar;

	// write data to the screen
	// '\t' separates values with tab characters
	// '\n' and 'endl' add end-of-line (return) characters
	cout<<"\nPumping Rate:\t\t"<<wd.PumpingRate<<endl;
	cout<<"Aquifer Thickness:\t"<<wd.Thickness<<endl;
	cout<<"Distance to Piezometer:\t"<<wd.Radius<<endl;
	cout<<"\nTrial\tTime\tDrawdown\n";
	for (counter = 0; counter < wd.MaxTrials; ++counter, ++LineCount)
	{
		cout<<(counter + 1)<<"\t"<<wd.Time[counter]<<"\t"<<
			wd.Drawdown[counter]<<endl;
		val = div(LineCount, 15);
		if (val.rem == 0) // LineCount = 10;
		{
			LineCount = 0;
			cout<<"\n\n<ENTER> to continue...";
			getch(); // get user keypress
		}
	} // end of loop
	cout<<endl;
} // end of function ViewData





void EnterTestData(void)
{
	// this gets the pumping rate, aquifer thickness, and distance to the
	// piezometer from the user via the keyboard
	cout<<"Enter the pumping rate"<<endl;
	cin>>wd.PumpingRate;
	cout<<"Enter the distance to the piezometer"<<endl;
	cin>>wd.Radius;
	cout<<"Enter the aquifer thickness"<<endl;
	cin>>wd.Thickness;
	cout<<endl;
} // end of function EnterTestData





void EnterTimeDrawdown(void)
{
	// this function gets the time and drawdown data from the user
	// through the keyboard

	int counter;
	cout<<"\nEnter the number of drawdown measurements (max of 30)"<<endl;
	cin>>wd.MaxTrials;

	// this loop gets time and drawdown from the user
	// unlike FORTRAN, C subscripts arrays starting with 0
	for (counter = 0; counter < wd.MaxTrials; counter = counter + 1)
	{
		cout<<"Enter time in days for trial "<<(counter + 1)<<endl;
		cin>>wd.Time[counter];
		cout<<"Enter drawdown for time "<<(wd.Time[counter])<<endl;
		cin>>wd.Drawdown[counter];
	} // end of loop
	cout<<endl;
} // end of function EnterTimeDrawdown





void ReviseTimeDrawdown(void)
{ // this function allows the user to change a time or drawdown value
	int Number;
	cout<<"\nEnter the number to change\n";
	cout<<"(First trial is number 1)\n";
	cin>>Number;
	cout<<"The old time is "<<wd.Time[Number-1]<<endl;
	cout<<"Enter the new time ";
	cin>>wd.Time[Number-1]; // subtract 1 to account for 0 based subscripts
	cout<<"The old drawdown is "<<wd.Drawdown[Number-1]<<endl;
	cout<<"Enter the new drawdown ";
	cin>>wd.Drawdown[Number+1];
	cout<<endl;
} // end of function ReviseTimeDrawdown



// This function adds a time and drawdown value to the end of the test
// It is useful for analyzing data as a test progresses
void AppendData(void)
{
	cout<<"\nLast time is "<<wd.Time[wd.MaxTrials-1]<<endl;
	cout<<"Last drawdown is "<<wd.Drawdown[wd.MaxTrials-1]<<endl;
	cout<<"Enter the new time value\n";
	cin>>wd.Time[wd.MaxTrials];
	cout<<"Enter the new drawdown value\n";
	cin>>(wd.Drawdown[wd.MaxTrials]);
	++wd.MaxTrials;
	cout<<"There are now "<<wd.MaxTrials<<" trials"<<endl;
	cout<<endl;
}



int WriteFile(void)
{ // this function writes the pump test data to a file
	// in a format compatible with Starpoint Software's Infinite ExtentTM
	// The function returns 1 if successful, 0 if there was a file writing error
	char FileName[MAXFILELENGTH];
	int counter;
	ofstream outf;

	// get the file name from the user
	cout<<"\nEnter the complete file name\n";
	cout<<"for example c:\\slope\\example1.in"<<endl;
	cin>>FileName;

	outf.open(FileName);
	outf<<"[IE 2.1]"<<endl;
	outf<<wd.Title<<endl;
	outf<<"Unused"<<endl;
	outf<<"Unused"<<endl;
	outf<<"Unused"<<endl;
	outf<<setprecision(6)<<setiosflags(ios::left|ios::fixed);
	outf<<"Discharge rate"<<endl;
	outf<<wd.PumpingRate<<endl;
	outf<<"Well radius"<<endl;
	outf<<"0.0"<<endl;
	outf<<"Distance to the piezometers"<<endl;
	outf<<setprecision(4)<<setw(12)<<"1"<<setw(12)<<wd.Radius<<
		setw(12)<<"0.0"<<setw(12)<<"0.0"<<setw(12)<<"0.0"<<endl;
	outf<<"Storativity"<<endl;
	outf<<"0.0"<<endl;
	outf<<"Aquifer thickness"<<endl;
	outf<<wd.Thickness<<endl;
	outf<<"Partial Penetration Parameters"<<endl;
	outf<<"0.0"<<"\t"<<"0.0"<<"\t"<<"0.0"<<"\t"<<
	 "0.0"<<"\t"<<"0.0"<<endl;
	outf<<"0.0"<<"\t"<<"0.0"<<"\t"<<"0.0"<<"\t"<<
	 "0.0"<<"\t"<<"0.0"<<endl;

	outf<<wd.MaxTrials<<endl;
	outf<<"time and drawdown"<<endl;
	counter = -1;
	start:++counter;
		outf<<setprecision(6)<<setw(10)<<wd.Time[counter]<<setw(10)<<
			"0.0"<<
			setw(10)<<wd.Drawdown[counter]<<setw(10)<<"0.0"<<
			setw(10)<<"0.0"<<setw(10)<<"0.0"<<endl;
	if (counter < wd.MaxTrials-1) goto start;

	if (outf.good())
	{
		cout<<"\nSuccessful file operation\n"<<endl;
		return(0);
	}
	cout<<"\nFaileure in file operation\n"<<endl;
	return (1);
} // end of function to write a data file





int ReadFile(void)
{ // this function reads a file of pump test data
	// created from te WriteFile function or in
	// Starpoint Software's Infinite Extent format
	// the function returns a 1 for success
	// and a 0 if there is a file reading error

	char Buffer[80], // used to read the unused text values in the file
			FileName[MAXFILELENGTH];
	float temp; // used to read in values used by Infinite Extent
							// but not this slope matching program
	int counter = 0;
	ifstream inf;

	cout<<"\nEnter the file name to read ";
	cin>>FileName;

	inf.open(FileName);
	inf.getline(Buffer, 80);
	inf.getline(wd.Title, 80);
	cout<<"Title:\t"<<wd.Title<<endl;
	inf.getline(Buffer, 80);
	inf.getline(Buffer, 80);
	inf.getline(Buffer, 80);
	inf>>Buffer;
	inf>>Buffer;
	inf>>wd.PumpingRate;
	inf>>Buffer;
	inf>>Buffer;
	inf>>temp;
	inf>>Buffer;
	inf>>Buffer;
	inf>>Buffer;
	inf>>Buffer;
	inf>>temp>>wd.Radius>>temp>>temp>>temp;
	inf>>Buffer;
	//cout<<Buffer<<endl;
	inf>>temp;
	//cout<<fStor<<endl;
	inf>>Buffer;
	inf>>Buffer;
	inf>>wd.Thickness;
	inf>>Buffer;
	inf>>Buffer;
	inf>>Buffer;
	inf>>temp>>temp>>temp>>temp>>temp;
	inf>>temp>>temp>>temp>>temp>>temp;
	inf>>wd.MaxTrials;
	cout<<"There are "<<wd.MaxTrials<<" trials"<<endl<<endl;
	inf>>Buffer;
	inf>>Buffer;
	inf>>Buffer;

	counter = 0;
	// read time and drawdown values
	while (wd.MaxTrials > 0 && counter < wd.MaxTrials && counter < MAX)
	{
		inf>>wd.Time[counter]>>temp>>wd.Drawdown[counter]>>temp>>temp>>temp;
		counter = counter + 1;
	} // end of while loop
	if (inf.good())
	{
		cout<<"\nSuccessful file operation\n"<<endl;
		return(1);
	}
	cout<<"\nFailure in file operation\n"<<endl;
	return (0);
} // end of ReadFile function



void WriteLongDoubleSlopeFile(void)
{
	int Base, Exponent;
	long double Wu, u, Slope;
	ofstream outf;
	char FileName[MAXFILELENGTH];
	cout<<"\nEnter the complete file name\n";
	cout<<"for example c:\\slope\\example1.txt"<<endl;
	cin>>FileName;

	outf.open(FileName);
	outf<<"\t1.0\t2.0\t3.0\t4.0\t5.0\t6.0\t7.0\t8.0\t9.0"<<endl;
	for (Exponent = 0; Exponent >= -10; --Exponent)
	{
		outf<<"10^"<<Exponent<<"\t";
		for (Base = 1; Base < 10; ++Base)
		{
			u = ((long double)(Base))*powl(10.0, (long double)Exponent);
			Wu = WellFunctionl(u);
			Slope = expl(-u)/Wu;
			outf<<Slope<<"\t";
		}
		outf<<endl;
	} // end of exponent loop
  cout<<"\nFile Written\n\n";
}


/*	This function creates a table of the Theis well function
	W(u) for u using the long double functions
*/

void WriteLongDoubleWellFunctionFile(void)
{
	int Base, Exponent;
	long double Wu, u;
	ofstream outf;
	char FileName[MAXFILELENGTH];
  cout<<"\nWrite Well Function File\n";
	cout<<"Enter the complete file name\n";
	cout<<"for example c:\\slope\\example1.txt"<<endl;
	cin>>FileName;

	outf.open(FileName);
	outf<<"\t1.0\t2.0\t3.0\t4.0\t5.0\t6.0\t7.0\t8.0\t9.0"<<endl;
	for (Exponent = 0; Exponent >= -10; --Exponent)
	{
		outf<<"10^"<<Exponent<<"\t";
		for (Base = 1; Base < 10; ++Base)
		{
			u = ((long double)(Base))*powl(10.0, (long double)Exponent);
			Wu = WellFunctionl(u);
			outf<<Wu<<"\t";
		}
		outf<<endl;
	} // end of exponent loop
  cout<<"\nFile Written\n\n";
}


void WriteSlopeFile(void)
{
	int Base, Exponent;
	double Wu, u, Slope;
	ofstream outf;
	char FileName[MAXFILELENGTH];
	cout<<"\nEnter the complete file name\n";
	cout<<"for example c:\\slope\\example1.txt"<<endl;
	cin>>FileName;

	outf.open(FileName);
	outf<<"\t1.0\t2.0\t3.0\t4.0\t5.0\t6.0\t7.0\t8.0\t9.0"<<endl;
	for (Exponent = 0; Exponent >= -10; --Exponent)
	{
		outf<<"10^"<<Exponent<<"\t";
		for (Base = 1; Base < 10; ++Base)
		{
			u = ((double)(Base))*pow(10.0, (double)Exponent);
			Wu = WellFunction(u);
			Slope = exp(-u)/Wu;
			outf<<Slope<<"\t";
		}
		outf<<endl;
	} // end of exponent loop
  cout<<"\nFile Written\n\n";
}


/*	This function creates a table of the Theis well function
	W(u) for u using the double functions
*/

void WriteWellFunctionFile(void)
{
	int Base, Exponent;
	double Wu, u;
	ofstream outf;
	char FileName[MAXFILELENGTH];
  cout<<"\nWrite Well Function File\n";
	cout<<"Enter the complete file name\n";
	cout<<"for example c:\\slope\\example1.txt"<<endl;
	cin>>FileName;

	outf.open(FileName);
	outf<<"\t1.0\t2.0\t3.0\t4.0\t5.0\t6.0\t7.0\t8.0\t9.0"<<endl;
	for (Exponent = 0; Exponent >= -10; --Exponent)
	{
		outf<<"10^"<<Exponent<<"\t";
		for (Base = 1; Base < 10; ++Base)
		{
			u = ((double)(Base))*pow(10.0, (double)Exponent);
			Wu = WellFunction(u);
			outf<<Wu<<"\t";
		}
		outf<<endl;
	} // end of exponent loop
  cout<<"\nFile Written\n\n";
}



main()
{
	int MenuSelection;

	// write introductory information to the screen
	cout<<"This program solves pump tests using"<<endl;
	cout<<"slope matching for either leaky"<<endl;
	cout<<"or confined aquifer.  Data is entered"<<endl;
	cout<<"by the user and can be saved to a file"<<endl;
	cout<<"or run from memory.  Output is written to a file"<<endl;
	cout<<"specified by the user.  All values should be"<<endl;
	cout<<"entered in consistent units\n"<<endl;
	// begin the menu loop
	do
	{
		// display menu options
		cout<<"Select an option\n";
		cout<<"1 < Enter test data\n";
		cout<<"2 < Enter time and drawdown data\n";
		cout<<"3 < Revise time and drawdown data\n";
		cout<<"4 < Append a time and drawdown measurement\n";
		cout<<"5 < View all data\n";
		cout<<"6 < Write test data to a file\n";
		cout<<"7 < Read test data from a file\n\n";
		cout<<"8 < Run the Theis slope matching solution for confined aquifers\n";
		cout<<"9 < Run the Hantush slope matching solution for leaky aquifers\n\n";
		cout<<"10 < Create a file of slope values using doubles\n";
	    cout<<"11 < Create a file of W(u) values using doubles\n";
		cout<<"12 < Create a file of slope values using long doubles\n";
    cout<<"13 < Create a file of W(u) values using long doubles\n";
		cout<<"14 < Exit the program\n";
		cout<<" -->";
		cin>>MenuSelection;

		// evaluate menu selection
		switch (MenuSelection)
		{
			case 1: // Enter test data
				EnterTestData();
				break;
			case 2: // Enter time and drawdown
				EnterTimeDrawdown();
				break;
			case 3: // Revise time and drawdown
				ReviseTimeDrawdown();
				break;
			case 4: // Append a time and drawdown value
				AppendData();
				break;
			case 5: // view all data
				ViewData();
				break;
			case 6: // Write test data to a file
				WriteFile();
				break;
			case 7: // Read test data from a file
				ReadFile();
				break;
			case 8: //  Run the Theis slope matching solution
				InitializeValues();
				TheisCalculator();
				WriteTheisFile();
				break;
			case 9: // Run the Hantush slope matching solution
				InitializeValues();
				HantushCalculator();
				WriteHantushFile();
				break;
			case 10:
				WriteSlopeFile();
				break;
			case 11:
				WriteWellFunctionFile();
				break;
  		case 12:
				WriteLongDoubleSlopeFile();
				break;
			case 13:
				WriteLongDoubleWellFunctionFile();
				break;
  	} // end of switch
	} // end of do..while
	while (MenuSelection != 14);

	cout<<"Goodbye";
	return(0);
}